var Event = function(id,eventdate,mileage,eventtype,value,remarks) {
	this.id = ko.observable(id);
	this.eventdate = ko.observable(eventdate);
	this.mileage = ko.observable(mileage);
	this.eventtype = ko.observable(eventtype);
	this.value = ko.observable(value);
	this.remarks = ko.observable(remarks);
};

var Vehicle = function(id,platenom,vehicletype,mileage) {
	this.id = ko.observable(id);
	this.platenom = ko.observable(platenom);
	this.vehicletype = ko.observable(vehicletype);
	this.mileage = ko.observable(mileage);
	this.events = ko.observableArray();
	this.servicemileage = ko.observable(mileage);
};

var FuelType = function(id,name,price) {
	this.id = ko.observable(id);
	this.name = ko.observable(name);
	this.price = ko.observable(price);
};

function EventModel() {
	var self = this;
	self.db = window.openDatabase("Database","1.0","VSLog DB",200000);
	if(!window.localStorage.getItem("gotdb")){
		self.db.transaction(function(tx){ self.initdb(tx) });
	};
	self.db.transaction(function(tx){ self.loadData(tx) });
	self.carserviceinterval = ko.observable(5000);
	self.bikeserviceinterval = ko.observable(2000);
	self.carrefuelinterval = ko.observable(200);
	self.bikerefuelinterval = ko.observable(100);
	self.platenom = ko.observable('None');
	self.fuels = ko.observableArray();

	self.fuelnameform = ko.observable();
	self.fuelpriceform = ko.observable();
	self.curfuel = ko.observable();

	self.vehicles = ko.observableArray();

	self.curpage = ko.observable('normal');

	self.newplatenom = ko.observable('');
	self.newvehicletype = ko.observable('Car');
	self.newmileage = ko.observable('');

	self.eventmileage = ko.observable('');
	self.eventtype = ko.observable('');
	self.eventdate = ko.observable('');
	self.eventvalue = ko.observable(0);
	self.eventremarks = ko.observable('');
	self.eventfuel = ko.observable();
	self.eventfuel(self.fuels()[0]);
	self.eventamount = ko.computed(function() {
		return self.eventvalue() ? (self.eventvalue() / self.eventfuel().price()).toFixed(2):(0).toFixed(2);
	});
	self.curevent = ko.observable();

	self.currentvehicle = ko.observable();

	self.initdb = function(tx) {
		tx.executeSql('drop table if exists vehicles');
		tx.executeSql('create table if not exists vehicles(id integer primary key autoincrement, platenom text, vehicletype text, mileage integer)');
		tx.executeSql('drop table if exists events');
		tx.executeSql('create table if not exists events(id integer primary key autoincrement, eventdate date, mileage integer, eventtype text, value real, remarks text, vehicle_id integer)');
		tx.executeSql('drop table if exists fueltype');
		tx.executeSql('create table if not exists fueltype(id integer primary key autoincrement, name text, price real)');
		tx.executeSql('insert into fueltype(name,price) values ("RON 95",1.9)');
		tx.executeSql('insert into fueltype(name,price) values ("RON 97",2.3)');
		window.localStorage.setItem("carserviceinterval",5000);
		window.localStorage.setItem("bikeserviceinterval",2000);
		window.localStorage.setItem("carrefuelinterval",500);
		window.localStorage.setItem("bikerefuelinterval",100);
		window.localStorage.setItem('gotdb',1);
	};

	self.resetdb = function() {
		self.db.transaction(function(tx){ self.initdb(tx) });
		self.db.transaction(function(tx){ self.loadData(tx) });
	};

	self.loadData = function(tx) {
		tx.executeSql('select * from fueltype',null,function(tx,results){
			for(var i=0;i < results.rows.length; i++){
				var row = results.rows.item(i);
				self.fuels.push(new FuelType(row.id,row.name,row.price));
			};
		});
		tx.executeSql('select * from vehicles',null,function(tx,results){
			for(var i=0;i < results.rows.length; i++){
				var row = results.rows.item(i);
				var curvehicle = new Vehicle(row.id,row.platenom,row.vehicletype,row.mileage);
				self.vehicles.push(curvehicle);
			};
			ko.utils.arrayForEach(self.vehicles(), function(curv) {
				tx.executeSql('select * from events where vehicle_id=? order by eventdate desc',[curv.id()],function(tx,eventresults){
					var smset = false;
					for(var j=0;j < eventresults.rows.length; j++){
						var eventrow = eventresults.rows.item(j);
						var newevent = new Event(eventrow.id,eventrow.eventdate,eventrow.mileage,eventrow.eventtype,eventrow.value,eventrow.remarks);
						if(!smset){
							if(eventrow.eventtype=='Service'){
								curv.servicemileage(eventrow.mileage);
								smset = true;
							}
						}
						curv.events.push(newevent);
					};
					if(self.platenom()=='None'){
						self.platenom(curv.platenom());
						self.currentvehicle(curv);
						self.updateTab();
					};
				});
			});
		});
		self.carserviceinterval(window.localStorage.getItem("carserviceinterval"));
		self.bikeserviceinterval(window.localStorage.getItem("bikeserviceinterval"));
		self.carrefuelinterval(window.localStorage.getItem("carrefuelinterval"));
		self.bikerefuelinterval(window.localStorage.getItem("bikerefuelinterval"));
	};

	self.togglesetting = function() {
		if(self.curpage()=='normal'){
			self.curpage('settings');
		}
		else{
			window.localStorage.setItem("carserviceinterval",self.carserviceinterval());
			window.localStorage.setItem("bikeserviceinterval",self.bikeserviceinterval());
			window.localStorage.setItem("carrefuelinterval",self.carrefuelinterval());
			window.localStorage.setItem("bikerefuelinterval",self.bikerefuelinterval());
			self.curpage('normal');
		};
	};

	self.fuelDialog = function(name,price) {
		self.fuelnameform(name);
		self.fuelpriceform(price);
		var match = ko.utils.arrayFirst(self.fuels(),function(item) {
			return self.fuelnameform()==item.name();
		});
		self.curfuel(match);
		$('#fuelform').modal('show');
	};

	self.deleteFuel = function() {
		self.db.transaction(
				function(tx){
					tx.executeSql('delete from fueltype where id=?',[self.id()],function(tx,result){
					});
				});
		self.fuels.remove(self.curfuel());
		$('#fuelform').modal('hide');
	};

	self.addFuel = function() {
		var match = ko.utils.arrayFirst(self.fuels(),function(item) {
			return self.fuelnameform()==item.name();
		});
		if(!match){
			var newfuel = new FuelType(0,self.fuelnameform(),self.fuelpriceform());
			self.db.transaction(
					function(tx){
						tx.executeSql('insert into fueltype(name,price) values (?,?)',[self.fuelnameform(),self.fuelpriceform()],function(tx,result){
							newfuel.id(result.insertId);
						});
					});
			self.fuels.push(newfuel);
		}
		else{
			match.price(self.fuelpriceform());
			self.db.transaction(
					function(tx){
						tx.executeSql('update fueltype set price=? where id=?',[match.price(),match.id()]);
					});
		}
		$('#fuelform').modal('hide');
	};

	self.showVehicle = function(platenom) {
		self.platenom(platenom);
		self.currentvehicle(ko.utils.arrayFirst(self.vehicles(),function(item){
			return item.platenom()==platenom;
		}));
		self.updateTab();
	};

	self.addVehicle = function(formElement) {
		var newvehicle = new Vehicle(0,self.newplatenom(),self.newvehicletype(),self.newmileage());
		self.db.transaction(
				function(tx){
					tx.executeSql('insert into vehicles(platenom,vehicletype,mileage) values (?,?,?)',[newvehicle.platenom(),newvehicle.vehicletype(),newvehicle.mileage()],function(tx,result){
						newvehicle.id(result.insertId);
					});
				});
		self.vehicles.push(newvehicle);
		self.newplatenom('');
		self.newvehicletype('Car');
		self.newmileage('');
	};

	self.addEvent = function() {
		var newevent = new Event(0,self.eventdate(),self.eventmileage(),self.eventtype(),self.eventvalue(),self.eventremarks());
		if(self.eventtype()=='Service'){
			self.currentvehicle().servicemileage(newevent.mileage());
		}
		self.db.transaction(
				function(tx){
					tx.executeSql('insert into events(eventdate,mileage,eventtype,value,remarks,vehicle_id) values (?,?,?,?,?,?)',[newevent.eventdate(),newevent.mileage(),newevent.eventtype(),newevent.value(),newevent.remarks(),self.currentvehicle().id()],function(tx,result){
						newevent.id(result.insertId);
					});
				});
		self.currentvehicle().events.unshift(newevent);
		self.currentvehicle().mileage(self.eventmileage());
		self.db.transaction(
				function(tx){
					tx.executeSql('update vehicle set mileage=? where id=?',[self.eventmileage(),self.currentvehicle().id()])
					});
		$('#eventdialog').modal('hide');
	};

	self.deleteEvent = function() {
		self.db.transaction(
				function(tx){
					tx.executeSql('delete from events where id=?',[self.curevent().id()])
					});
		self.currentvehicle().events.remove(self.curevent());
		$('#displayevent').modal('hide');
	};

	self.displayEvent = function(curevent) {
		self.curevent(curevent);
		self.eventmileage(curevent.mileage());
		self.eventvalue(curevent.value());
		self.eventtype(curevent.eventtype());
		self.eventdate(curevent.eventdate());
		self.eventremarks(curevent.remarks());
		$('#displayevent').modal('show');
	};

	self.eventDialog = function(eventtype) {
		if(eventtype=='Service'){
			if(self.currentvehicle().vehicletype()=='Car'){
				self.eventmileage(parseInt(self.currentvehicle().servicemileage()) + parseInt(self.carserviceinterval()));
			}
			else{
				self.eventmileage(parseInt(self.currentvehicle().servicemileage()) + parseInt(self.bikeserviceinterval()));
			};
		}
		else{
			if(self.currentvehicle().vehicletype()=='Car'){
				self.eventmileage(parseInt(self.currentvehicle().mileage()) + parseInt(self.carrefuelinterval()));
			}
			else{
				self.eventmileage(parseInt(self.currentvehicle().mileage()) + parseInt(self.bikerefuelinterval()));
			};
		};
		self.eventvalue(0);
		self.eventfuel(self.fuels()[0]);
		self.eventtype(eventtype);
		self.eventdate(new Date());
		self.eventremarks('');
		$('#eventdialog').modal('show');
	};

	self.updateTab = function() {
		$('.nav li').each(function(){
			if($(this).data('plate')==self.platenom()){
				$(this).addClass('active');
			}
			else{
				$(this).removeClass('active');
			};
		});
	};
	
};

eventmodel = new EventModel();
eventmodel.updateTab();

ko.applyBindings(eventmodel);
